
import java.util.Iterator;

/**
 * This class generates a random terrain.  It iteratively splits the terrain
 * into smaller pieces and randomly displaces the height of each segment.
 * <p/>
 * More details about the random fractal terrain generation algorithm can be
 * found here: http://www.gameprogrammer.com/fractal.html
 *
 * @author Michael Yu
 */
public class TerrainGenerator {

    /**
     * Generate a completely flat terrain
     *
     * @param width  is the width of the generated terrain
     * @param height is the height of the generated terrain
     * @return a flat terrain with the given width and height
     */
    public static Terrain generateFlat(double width, double height) {
        Terrain terrain = new Terrain(new double[]{height}, width);
        terrain.getEncompassingSegment(0.0).setLandingPad(true);
        terrain.getEncompassingSegment(0.0).setMultiplier(1);
        return terrain;
    }

    /**
     * Generate a random terrain
     *
     * @param width      is the width of the generated terrain
     * @param low        is the minimum altitude of the terrain
     * @param high       is the maximum altitude of the terrain
     * @param iterations is the number of multiply we subdivide the terrain
     * @param H          is the smoothing factor.  It must be between 0 and 1.
     *                   The greater the value, the smoother the generated
     *                   terrain will be.
     * @return the randomly generated terrain
     */
    public static Terrain generateRandom(double width, double low, double high,
            int iterations, double H) {

        double[] heightMap = new double[1 << iterations];
        double range = 1000.0;
        for (int i = 0; i < iterations; i++) {
            range *= H;
            int stepSize = (1 << iterations) / (1 << i);
            for (int j = 0; j < heightMap.length; j += stepSize) {
                heightMap[j + stepSize / 2] = (heightMap[j] +
                        heightMap[(j + stepSize) % heightMap.length]) / 2;
                heightMap[j + stepSize / 2] += (Math.random() - 0.5) / 0.5 *
                        range;
            }
        }

        Terrain result = new Terrain(heightMap, width);
        normalize(result, low, high);
        
        for(int i=0; i<result.getSegments().size(); i++) {
            System.out.print(((TerrainSegment)result.getSegments().get(i)).getRightEndPoint().getY() + ",");
        }
        
        createLandingPads(result);
        
        return result;
    }
    
    public static Terrain generateLoaded(double width, double[] heightMap, 
            int[] longPads, int[] shortPads) {
        
        Terrain result = new Terrain(heightMap, width);
        
        for(int i=0; i<longPads.length; i++) {
            createLongPad(result, longPads[i]);
        }
        
        for(int i=0; i<shortPads.length; i++) {
            createShortPad(result, shortPads[i]);
        }
        return result;
    }

    /**
     * Normalize the terrain so that its height ranges from [low, high]
     *
     * @param terrain is the terrain to be normalized
     * @param low     is the minimum altitude of the terrain
     * @param high    is the maximum altitude of the terrain
     */
    private static void normalize(Terrain terrain, double low, double high) {
        double maxHeight = maxHeight(terrain);
        double minHeight = minHeight(terrain);

        for (Iterator it = terrain.getSegments().iterator(); it.hasNext();) {
            TerrainSegment segment = (TerrainSegment) it.next();

            // Translate the terrain so that the lowest height is 0
            Vect2D translation = new Vect2D(0, -minHeight);
            segment.setLeftEndPoint(segment.getLeftEndPoint().add(translation));
            segment.setRightEndPoint(
                    segment.getRightEndPoint().add(translation));

            // Scale the terrain so that the highest height is high-low
            Matrix scaleY = new Matrix(1, 0, 0,
                    (high - low) / (maxHeight - minHeight));
            segment.setLeftEndPoint(scaleY.times(segment.getLeftEndPoint()));
            segment.setRightEndPoint(scaleY.times(segment.getRightEndPoint()));

            // Translate the terrain so that the lowest height is 'low'
            Vect2D translation2 = new Vect2D(0, low);
            segment.setLeftEndPoint(
                    segment.getLeftEndPoint().add(translation2));
            segment.setRightEndPoint(
                    segment.getRightEndPoint().add(translation2));
        }
    }

    /**
     * Return the maximum height of the given terrain
     *
     * @param terrain is the terrain to query
     * @return the maximum height of the given terrain
     */
    private static double maxHeight(Terrain terrain) {
        double soFar = Double.NEGATIVE_INFINITY;

        for (Iterator it = terrain.getSegments().iterator(); it.hasNext();) {
            TerrainSegment segment = (TerrainSegment) it.next();
            if (segment.getLeftEndPoint().getY() > soFar) {
                soFar = segment.getLeftEndPoint().getY();
            }
        }

        return soFar;
    }

    /**
     * Return the minimum height of the given terrain
     *
     * @param terrain is the terrain to query
     * @return the minimum height of the given terrain
     */
    private static double minHeight(Terrain terrain) {
        double soFar = Double.POSITIVE_INFINITY;

        for (Iterator it = terrain.getSegments().iterator(); it.hasNext();) {
            TerrainSegment segment = (TerrainSegment) it.next();
            if (segment.getLeftEndPoint().getY() < soFar) {
                soFar = segment.getLeftEndPoint().getY();
            }
        }

        return soFar;
    }

    /**
     * Given a terrain, randomly select a few segments and transform them into
     * landing pads
     *
     * @param terrain is the terrain which we will add landing pads to
     */
    private static void createLandingPads(Terrain terrain) {
        createRandomShortLandingPad(terrain);

        int iterations = terrain.getSegments().size() / 30;
        for (int i = 0; i < iterations; i++) {
            createRandomShortLandingPad(terrain);
            createRandomLongLandingPad(terrain);
        }
    }

    /**
     * Given a terrain, create a single, short landing pad.  A short landing pad
     * comprises one terrain segment.
     *
     * @param terrain is the terrain to add the landing pad to
     */
    private static void createRandomShortLandingPad(Terrain terrain) {
        int numSegments = terrain.getSegments().size();
        int originalIndex = (int) (numSegments * Math.random());

        for (int i = (originalIndex + 1) % numSegments;
                i != originalIndex; i = (i + 1) % numSegments) {

            TerrainSegment segment = (TerrainSegment) terrain.getSegments()
                    .get(i);
            TerrainSegment leftNeighbor = (TerrainSegment) terrain.getSegments()
                    .get((i - 1 + numSegments) % numSegments);
            TerrainSegment rightNeighbor = (TerrainSegment) terrain.getSegments()
                    .get((i + 1) % numSegments);

            if (!leftNeighbor.isLandingPad() && !segment.isLandingPad() &&
                    !rightNeighbor.isLandingPad()) {

                double landingPadHeight = segment.getLeftEndPoint().getY();

                // Change the right end point's height to make the landing pad
                // horizontal
                segment.setRightEndPoint(new Vect2D(
                        segment.getRightEndPoint().getX(), landingPadHeight));

                // The right neighbor's left end point needs to change too!
                rightNeighbor.setLeftEndPoint(new Vect2D(
                        rightNeighbor.getLeftEndPoint().getX(),
                        landingPadHeight));

                segment.setLandingPad(true);
                segment.setMultiplier(SHORT_LANDING_PAD_MULTIPLIER);
                return;
            }
        }
    }
    
    private static void createShortPad(Terrain terrain, int index) {
        int numSegments = terrain.getSegments().size();
        TerrainSegment segment = (TerrainSegment) terrain.getSegments().get(index);
        TerrainSegment rightNeighbor = (TerrainSegment) terrain.getSegments().get((index + 1) % numSegments);
        double landingPadHeight = segment.getLeftEndPoint().getY();
        
        // Change the right end point's height to make the landing pad
        // horizontal
        segment.setRightEndPoint(new Vect2D(
                segment.getRightEndPoint().getX(), landingPadHeight));

        // The right neighbor's left end point needs to change too!
        rightNeighbor.setLeftEndPoint(new Vect2D(
                rightNeighbor.getLeftEndPoint().getX(),
                landingPadHeight));

        segment.setLandingPad(true);
        segment.setMultiplier(SHORT_LANDING_PAD_MULTIPLIER);
    }


    /**
     * Given a terrain, create a single, long landing pad.  A long landing pad
     * comprises two terrain segments.
     *
     * @param terrain is the terrain to add the landing pad to
     */
    private static void createRandomLongLandingPad(Terrain terrain) {
        int numSegments = terrain.getSegments().size();
        int originalIndex = (int) (numSegments * Math.random());

        for (int i = (originalIndex + 1) % numSegments;
                i != originalIndex; i = (i + 1) % numSegments) {

            if (i + 3 >= numSegments) {
                continue;
            }

            TerrainSegment segment = (TerrainSegment) terrain.getSegments()
                    .get(i);
            TerrainSegment leftNeighbor = (TerrainSegment) terrain.getSegments()
                    .get((i - 1 + numSegments) % numSegments);
            TerrainSegment rightNeighbor = (TerrainSegment) terrain.getSegments()
                    .get((i + 1) % numSegments);
            TerrainSegment nextRightNeighbor = (TerrainSegment) terrain.getSegments()
                    .get((i + 2) % numSegments);

            if (!leftNeighbor.isLandingPad() && !segment.isLandingPad() &&
                    !rightNeighbor.isLandingPad() &&
                    !nextRightNeighbor.isLandingPad()) {

                double landingPadHeight = segment.getLeftEndPoint().getY();

                // Remove the segment's right neighbor
                terrain.getSegments().remove((i + 1) % numSegments);

                // Change the right end point of the segment so that it extends
                // horizontally to the old right neighbor's right end point.
                // Also, make the segment horizintal.
                segment.setRightEndPoint(new Vect2D(
                        rightNeighbor.getRightEndPoint().getX(),
                        landingPadHeight));

                // The new right neighbor's left end point needs to change to
                // match the height of the segment's right end point
                nextRightNeighbor.setLeftEndPoint(new Vect2D(
                        nextRightNeighbor.getLeftEndPoint().getX(),
                        landingPadHeight));

                segment.setLandingPad(true);
                segment.setMultiplier(LONG_LANDING_PAD_MULTIPLIER);
                return;
            }
        }
    }
    
    private static void createLongPad(Terrain terrain, int index) {
        int numSegments = terrain.getSegments().size();
        TerrainSegment segment = (TerrainSegment) terrain.getSegments().get(index);
        TerrainSegment rightNeighbor = (TerrainSegment) terrain.getSegments().get((index + 1) % numSegments);
        TerrainSegment nextRightNeighbor = (TerrainSegment) terrain.getSegments().get((index + 2) % numSegments);
        double landingPadHeight = segment.getLeftEndPoint().getY();
        
        terrain.getSegments().remove((index+1) % numSegments);
        // Change the right end point of the segment so that it extends
        // horizontally to the old right neighbor's right end point.
        // Also, make the segment horizintal.
        segment.setRightEndPoint(new Vect2D(
                rightNeighbor.getRightEndPoint().getX(),
                landingPadHeight));

        // The new right neighbor's left end point needs to change to
        // match the height of the segment's right end point
        nextRightNeighbor.setLeftEndPoint(new Vect2D(
                nextRightNeighbor.getLeftEndPoint().getX(),
                landingPadHeight));

        segment.setLandingPad(true);
        segment.setMultiplier(LONG_LANDING_PAD_MULTIPLIER);
        
    }

    // The score multiplier for a short landing pad
    private static final int SHORT_LANDING_PAD_MULTIPLIER = 2;
    // The score multiplier for a long landing pad
    private static final int LONG_LANDING_PAD_MULTIPLIER = 1;
}
