import java.awt.*;

/**
 * A terrain segment
 *
 * @author Michael Yu
 */
public class TerrainSegment {

    /**
     * Construct a new terrain segment
     *
     * @param leftEndPoint  is the left end point of the line segement
     * @param rightEndPoint is the right end point of the line segement
     * @param landingPad    is true if this segment is a landing pad
     * @param multiplier    is the score multiplier for this landing pad
     */
    public TerrainSegment(Vect2D leftEndPoint, Vect2D rightEndPoint,
            boolean landingPad, int multiplier) {
        this.leftEndPoint = leftEndPoint;
        this.rightEndPoint = rightEndPoint;
        this.landingPad = landingPad;
        this.multiplier = multiplier;
    }

    /**
     * Construct a new terrain segment that is not a landing pad
     *
     * @param leftEndPoint  is the left end point of the line segement
     * @param rightEndPoint is the right end point of the line segement
     */
    public TerrainSegment(Vect2D leftEndPoint, Vect2D rightEndPoint) {
        this(leftEndPoint, rightEndPoint, false, 0);
    }

    /**
     * Get the right end point of the line segment
     *
     * @return the right end point of the line segment
     */
    public Vect2D getRightEndPoint() {
        return rightEndPoint;
    }

    /**
     * Change the right end point of the line segment
     *
     * @param rightEndPoint is the new right end point of the line segment
     */
    public void setRightEndPoint(Vect2D rightEndPoint) {
        this.rightEndPoint = rightEndPoint;
    }

    /**
     * Get the left end point of the line segment
     *
     * @return the left end point of the line segment
     */
    public Vect2D getLeftEndPoint() {
        return leftEndPoint;
    }

    /**
     * Change the left end point of the line segment
     *
     * @param leftEndPoint is the new left end point of the line segment
     */
    public void setLeftEndPoint(Vect2D leftEndPoint) {
        this.leftEndPoint = leftEndPoint;
    }

    /**
     * Is this terrain segment a landing pad?
     *
     * @return true if this line segment is a landing pad, false otherwise
     */
    public boolean isLandingPad() {
        return landingPad;
    }

    /**
     * Toggle whether this terrain segment is a landing pad or not
     *
     * @param landingPad should be true if this terrain segment should become a
     *                   landing pad.  Should be false otherwise
     */
    public void setLandingPad(boolean landingPad) {
        this.landingPad = landingPad;
    }

    /**
     * Get the score multiplier for this terrain segment (only valid if this
     * segment is a landing pad)
     *
     * @return the multiplier for this landing pad
     */
    public int getMultiplier() {
        return multiplier;
    }

    /**
     * Change the multiplier score for this landing pad
     *
     * @param multiplier is the new multiplier for this landing pad
     */
    public void setMultiplier(int multiplier) {
        this.multiplier = multiplier;
    }

    /**
     * Draw the terrain segment and score multiplier text
     *
     * @param g is the graphics context to draw to
     */
    public void draw(Graphics g) {
        // Draw the multiplier text
        if (isLandingPad() && TerrainSegment.isMultiplierDisplayed()) {
            drawMultiplierText(g);
        }

        // Draw the terrain segment
        drawTerrainSegment(g);
    }

    /**
     * Draw the score multiplier text
     *
     * @param g is the graphics context to draw to
     */
    public void drawMultiplierText(Graphics g) {
        GameCanvas canvas = LunarLanderGame.canvas;
        double worldWidth = LunarLanderGame.moon.getWorldWidth();

        double llx = canvas.getLLX();
        double lly = canvas.getLLY();
        double urx = canvas.getURX();
        double ury = canvas.getURY();

        double centerX = leftEndPoint.getX() +
                (rightEndPoint.getX() - leftEndPoint.getX()) / 2;
        int x1 = (int) ((double) canvas.getWidth() *
                ((centerX - llx) / (urx - llx)));
        int x2 = (int) ((double) canvas.getWidth() *
                ((centerX - worldWidth - llx) / (urx - llx)));
        int x3 = (int) ((double) canvas.getWidth() *
                ((centerX + worldWidth - llx) / (urx - llx)));

        double centerY = leftEndPoint.getY() +
                (rightEndPoint.getY() - leftEndPoint.getY()) / 2;
        int y = (int) ((double) canvas.getHeight() *
                (1.0 - (centerY - lly) / (ury - lly)));

        Font f = new Font("Courier", Font.PLAIN,
                Math.min(12, canvas.getHeight()));
        g.setFont(f);
        int strWidth = g.getFontMetrics().stringWidth(multiplier + "X");

        g.setColor(Color.white);
        g.drawString(multiplier + "X", x1 - strWidth / 2, y + f.getSize() + 3);
        g.drawString(multiplier + "X", x2 - strWidth / 2, y + f.getSize() + 3);
        g.drawString(multiplier + "X", x3 - strWidth / 2, y + f.getSize() + 3);
    }

    /**
     * Draw the terrain segment
     *
     * @param g is the graphics context to draw to
     */
    public void drawTerrainSegment(Graphics g) {
        GameCanvas canvas = LunarLanderGame.canvas;
        double worldWidth = LunarLanderGame.moon.getWorldWidth();

        double llx = canvas.getLLX();
        double lly = canvas.getLLY();
        double urx = canvas.getURX();
        double ury = canvas.getURY();

        g.setColor(isLandingPad() ? Color.yellow : Color.white);

        int x1 = (int) ((double) canvas.getWidth() *
                (leftEndPoint.getX() - llx) / (urx - llx));
        int y1 = (int) ((double) canvas.getHeight() *
                (1.0 - (leftEndPoint.getY() - lly) / (ury - lly)));
        int x2 = (int) ((double) canvas.getWidth() *
                (rightEndPoint.getX() - llx) / (urx - llx));
        int y2 = (int) ((double) canvas.getHeight() *
                (1.0 - (rightEndPoint.getY() - lly) / (ury - lly)));
        g.drawLine(x1, y1, x2, y2);

        x1 = (int) ((double) canvas.getWidth() *
                (leftEndPoint.getX() - worldWidth - llx) / (urx - llx));
        x2 = (int) ((double) canvas.getWidth() *
                (rightEndPoint.getX() - worldWidth - llx) / (urx - llx));
        g.drawLine(x1, y1, x2, y2);

        x1 = (int) ((double) canvas.getWidth() *
                (leftEndPoint.getX() + worldWidth - llx) / (urx - llx));
        x2 = (int) ((double) canvas.getWidth() *
                (rightEndPoint.getX() + worldWidth - llx) / (urx - llx));
        g.drawLine(x1, y1, x2, y2);
    }

    /**
     * Should all terrain segments draw the score multiplier to the screen?
     *
     * @return true if the multiplier should be drawn on the screen
     */
    public static boolean isMultiplierDisplayed() {
        return multiplierDisplayed;
    }

    /**
     * Change whether all terrain segments should draw the score multiplier on
     * screen
     *
     * @param multiplierDisplayed is true if the multiplier should be drawn on
     *                            screen, false otherwise
     */
    public static void setMultiplierDisplayed(boolean multiplierDisplayed) {
        TerrainSegment.multiplierDisplayed = multiplierDisplayed;
    }

    private Vect2D leftEndPoint;  // The left end point of this line segment
    private Vect2D rightEndPoint; // The right end point of this line segment
    private boolean landingPad;   // Is this terrain segment a landing pad?
    private int multiplier;       // The score multipler for this landing pad

    private static boolean multiplierDisplayed; // Should all terrain segments
                                                // draw the score multiplier?
}
